-- M.Ind.applescript
-- M.Ind

(*

    M.Ind is a program to ease the creation of armies for the AT-43 wargame by RACKHAM.
    Copyright (C) 2007 Gurth <gurth@xs4all.nl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*)

-- M.Ind version alpha 0.1.0
--  Created by Gurth on 23-7-07.

property allowUnderstrengthUnits : false
property htmlFlavor : "xhtml1" --possible values: "html", "html32", "html401", "xhtml1"; anything else is seen as equivalent to "html"
property lineEnding : ASCII character 10
property MIndVersion : "beta 0.1.0"
property MIndVersionHTML : "&beta; 0.1.0"
property MIndWebSite : "http://www.xs4all.nl/~gurth/at-43/"
property playWarningSound : true
property saveFileName : ""
property toLowerCase : 0
property toUpperCase : 1
property warningSound : "Basso"

on action theObject
	if name of theObject is "Maximum AP" then
		checkAP()
	end if
end action

on activated theObject
	
	show window "main"
	
end activated

on alert ended theObject with reply withReply
	if button returned of withReply = (localized string "Save") then
		saveCompany()
		resetCompany()
	else if button returned of withReply = (localized string "Don't Save") then
		resetCompany()
	end if
end alert ended

on awake from nib theObject
	if name of theObject is "Main" then
		tell window "Main" to center
		createToolBar(theObject)
	else if name of theObject is "Preferences" then
		tell window "Preferences" to center
		
		tell tab view item "General" of tab view "Preferences" of theObject
			repeat with s in {"Basso", "Blow", "Bottle", "Frog", "Funk", "Glass", "Hero", "Ping", "Pop", "Purr", "Sosumi", "Submarine", "Tink"}
				make new menu item at end of menu items of menu of popup button "Warning Sounds" with properties {title:s, name:s}
			end repeat
			delete first menu item of menu of popup button "Warning Sounds"
		end tell
	end if
end awake from nib

on changed theObject
	if name of theObject is "Fighters" or name of theObject is "Officers" or name of theObject is "SpecialWeapons" or name of theObject is "Specialists" then
		syncTextAndStepper((name of theObject) as string, false)
	else if name of theObject is "Maximum AP" then
		checkAP()
	else if name of theObject is "Unit Background" then
		
		tell table view "Company" of scroll view "Company" of view "Company" of split view "Vertical" of window "Main"
			set sel to selected data item
			set contents of data cell "UnitBackground" of sel to (contents of text view "Unit Background" of scroll view "Unit Background" of window "Unit Background" of me) as string
		end tell
		
		set document edited of window "Main" to true
	end if
end changed

on choose menu item theObject
	if name of theObject is "About" then
		tell window "About" to center
		show window "About"
	else if name of theObject is "Add" then
		if selected data items of outline view "Available Units" of scroll view "Units" of split view "Horizontal" of split view "Vertical" of window "Main"  {} then
			addUnitToCompany()
		else
			addOptionToSelectedUnit()
		end if
	else if name of theObject is "Armies" then
		setUnitImage((resource path of main bundle) & "/" & title of current menu item of popup button "Armies" of window "Main" & ".png", "")
		showUnits(title of current menu item of popup button "Armies" of window "Main")
	else if name of theObject is "Edit Background" then
		if visible of window "Unit Background" is false then
			show window "Unit Background"
		else
			close window "Unit Background"
		end if
	else if name of theObject is "Export HTML" then
		exportCompany("html")
	else if name of theObject is "Export Text" then
		exportCompany("txt")
	else if name of theObject is "Import" then
		importCompany()
	else if name of theObject is "Maximize Unit" then
		maximizeOrStandardUnit(true)
	else if name of theObject is "New" then
		newCompany()
	else if name of theObject is "Open" then
		openCompany()
	else if name of theObject is "Save" then
		saveCompany()
	else if name of theObject is "Save As" then
		saveCompanyAs()
	else if name of theObject is "Standard Unit" then
		maximizeOrStandardUnit(false)
	else if name of theObject is "Remove Unit" then
		RemoveUnitFromCompany()
	else if name of theObject is "Show Unit Card" then
		if visible of window "Unit Card" is false then
			show window "Unit Card"
		else
			close window "Unit Card"
		end if
	else if name of theObject is "Warning Sounds" then
		set warningSound to title of current menu item of theObject
		play (load sound warningSound)
	end if
end choose menu item

on clicked theObject
	if name of theObject is "Add" then
		if selected data items of outline view "Available Units" of scroll view "Units" of split view "Horizontal" of split view "Vertical" of window "Main"  {} then
			addUnitToCompany()
		else
			addOptionToSelectedUnit()
		end if
	else if name of theObject is "Allow Understrength Units" then
		if state of theObject is 1 then
			set allowUnderstrengthUnits to true
		else
			set allowUnderstrengthUnits to false
		end if
	else if name of theObject is "AP Stepper" then
		set contents of text field "Maximum AP" of window "Main" to contents of stepper "AP Stepper" of window "Main"
		checkAP()
	else if name of theObject is "Available Units" then
		getUnitData(contents of data cell "Unit" of selected data item of theObject)
		set title of menu item "Add" of menu "Unit" of main menu to localized string "Add Unit to Company"
	else if name of theObject is "Company" then
		try
			setUnitMembers(selected data item of theObject)
			--selectUnit(selected data item of theObject)
			set selected data items of outline view "Available Units" of scroll view "Units" of split view "Horizontal" of split view "Vertical" of window "Main" to {}
			showOptions(findDataItemForUnit(contents of data cell "Unit" of selected data item of theObject))
			set title of menu item "Add" of menu "Unit" of main menu to localized string "Add Option to Selected Unit"
			enableRemoveButtonAndMenu(true)
		end try
	else if name of theObject is "Fighters" or name of theObject is "Officers" or name of theObject is "SpecialWeapons" or name of theObject is "Specialists" then
		syncTextAndStepper((name of theObject) as string, true)
	else if name of theObject is "html" or name of theObject is "html32" or name of theObject is "html401" or name of theObject is "xhtml1" then
		set htmlFlavor to (name of theObject) as string
	else if name of theObject is "Maximize Unit" then
		maximizeOrStandardUnit(true)
	else if name of theObject is "Warning Sound" then
		if state of theObject is 1 then
			set playWarningSound to true
			play (load sound warningSound)
		else
			set playWarningSound to false
		end if
		tell tab view item "General" of tab view "Preferences" of window "Preferences"
			set enabled of popup button "Warning Sounds" to playWarningSound
			set enabled of button "Test Warning Sound" to playWarningSound
		end tell
	else if name of theObject is "Remove Unit" then
		RemoveUnitFromCompany()
	else if name of theObject is "Standard Unit" then
		maximizeOrStandardUnit(false)
	else if name of theObject is "Test Warning Sound" then
		play (load sound warningSound)
	else if name of theObject is "Unit Picture" then
		show window "Unit Card"
	else if name of theObject is "Windows Text Files" then
		if state of theObject is 1 then
			set lineEnding to (ASCII character 13) & (ASCII character 10)
		else
			set lineEnding to ASCII character 10
		end if
	end if
end clicked

on clicked toolbar item theObject
	if identifier of theObject is "background item identifier" then
		if visible of window "Unit Background" is false then
			show window "Unit Background"
		else
			close window "Unit Background"
		end if
	else if identifier of theObject is "card item identifier" then
		if visible of window "Unit Card" is false then
			show window "Unit Card"
		else
			close window "Unit Card"
		end if
	else if identifier of theObject is "export html item identifier" then
		exportCompany("html")
	else if identifier of theObject is "export text item identifier" then
		exportCompany("txt")
	else if identifier of theObject is "info item identifier" then
		tell drawer "Information" of window "Main"
			if state is drawer opened or state is drawer opening then
				close drawer
			else
				open drawer
			end if
		end tell
	else if identifier of theObject is "warning item identifier" then
		selectIllegalUnits()
	end if
end clicked toolbar item

on column clicked theObject table column tableColumn
	
	-- Sorting a table or outline view does not work, even though it follows Apple's example to the letter
	
	-- Get the data source of the table view
	set theDataSource to data source of theObject
	
	-- Get the identifier of the clicked table column;
	--  you can instead use the name of the column
	set theColumnIdentifier to identifier of tableColumn
	
	-- Get the current sort column of the data source
	set theSortColumn to sort column of theDataSource
	
	-- If the current sort column is not the same as the clicked  column
	--   then switch the sort column
	if (name of theSortColumn) is not equal to theColumnIdentifier then
		set the sort column of theDataSource to data column theColumnIdentifier of theDataSource
	else
		-- Otherwise change the sort order
		if sort order of theSortColumn is ascending then
			set sort order of theSortColumn to descending
		else
			set sort order of theSortColumn to ascending
		end if
	end if
	
	-- Update the table view (so it will be redrawn)
	update theObject
end column clicked

on double clicked theObject
	if name of theObject is "Available Units" then
		addUnitToCompany() -- doesn't work: double-click doesn't seem to register while program is busy retrieving unit options
	end if
end double clicked

on keyboard up theObject event theEvent
	if name of theObject is "Available Units" then
		getUnitData(contents of data cell "Unit" of selected data item of theObject)
	else if name of theObject is "Company" then
		getUnitData(contents of data cell "Unit" of selected data item of theObject)
		--setUnitMembers(selected data item of theObject)
	end if
end keyboard up

on launched theObject
	
	if name of theObject is name of me then
		
		enableAddButtonAndMenu(false)
		enableRemoveButtonAndMenu(false)
		
		-- Create the data columns for unit data
		tell data source "unitDataSource"
			
			repeat with columnName in {"Army", "Unit", "EnglishUnitName", "Type", "AP", "APperFighter", "StandardNumber", "MaximumNumber", "NumbersBonus", "APperFighter", "StndSpecialists", "MaxSpecialists", "StndSpecialWeapons", "MaxSpecialWeapons", "StndOfficers", "MaxOfficers", "IsOfficer", "AppliesToAll"}
				make new data column at end of data columns with properties {name:columnName}
			end repeat
		end tell
		
		-- Set up categories
		set parentItem to make new data item at end of data items of data source "unitDataSource"
		set contents of data cell "Unit" of parentItem to localized string "Heroes"
		set contents of data cell "EnglishUnitName" of parentItem to "Heroes"
		
		set parentItem to make new data item at end of data items of data source "unitDataSource"
		set contents of data cell "Unit" of parentItem to localized string "Infantry"
		set contents of data cell "EnglishUnitName" of parentItem to "Infantry"
		
		set parentItem to make new data item at end of data items of data source "unitDataSource"
		set contents of data cell "Unit" of parentItem to localized string "Combat Striders"
		set contents of data cell "EnglishUnitName" of parentItem to "Combat Striders"
		
		set parentItem to make new data item at end of data items of data source "unitDataSource"
		set contents of data cell "Unit" of parentItem to localized string "Vehicles"
		set contents of data cell "EnglishUnitName" of parentItem to "Vehicles"
		
		
		-- Create the data columns for options data
		tell data source "optionsDataSource"
			repeat with columnName in {"Icon", "Unit", "OptionName", "AP", "APperFighter", "FixedAP", "NumbersBonus", "IsOfficer", "IsSpecialWeapon", "IsSpecialist", "AppliesToAll"}
				make new data column at end of data columns with properties {name:columnName}
			end repeat
		end tell
		
		
		-- Create the data columns for company data
		tell data source "companyDataSource"
			repeat with columnName in {"Unit", "Warning", "AP", "BaseAP", "APperFighter", "FixedAP", "OptionsAP", "AppliesToAll", "Options", "Type", "Fighters", "MinFighters", "MaxFighters", "Officers", "MinOfficers", "MaxOfficers", "NumbersBonus", "SpecialWeapons", "MinSpecialWeapons", "MaxSpecialWeapons", "Specialists", "MinSpecialists", "MaxSpecialists", "UnitBackground"}
				make new data column at end of data columns with properties {name:columnName, sort order:ascending, sort type:alphabetical, sort case sensitivity:case sensitive}
			end repeat
			set sort column to data column "Unit"
			set sorted to true
		end tell
		set data source of outline view "Company" of scroll view "Company" of view "Company" of split view "Vertical" of window "Main" to data source "companyDataSource"
		
		
		loadArmies()
		showUnits(title of current menu item of popup button "Armies" of window "Main")
	end if
	
end launched

on panel ended theObject with result withResult
	
	if withResult is 1 then
		set fileName to path name of save panel
		if (characters -1 thru -5 of fileName) as string = ".mind" then
			set saveFileName to path name of save panel
			writeCompanyToFile()
		else if (characters -1 thru -5 of fileName) as string = ".html" then
			writeCompanyToHTML(fileName)
		else
			writeCompanyToText(fileName)
		end if
	end if
	
end panel ended

on should close theObject
	
	set allowClose to true
	if name of theObject is "Main" then
		if document edited of window "Main" = true then
			set theReply to display alert (localized string "Do you want to save the changes you made to the company?") default button (localized string "Save") alternate button (localized string "Cancel") other button (localized string "Don't Save") message (localized string "Your changes will be lost if you don't save them.")
			if button returned of theReply = (localized string "Save") then
				saveCompany()
				resetCompany()
			else if button returned of theReply = (localized string "Cancel") then
				set allowClose to false
			else
				resetCompany()
			end if
		end if
	end if
	
	return allowClose
	
end should close

on will close theObject
	if name of theObject is "Information" then
		set title of menu item "Show Information" of menu "Unit" of main menu to localized string "Show Information"
	else if name of theObject is "Unit Card" then
		set title of menu item "Show Unit Card" of menu "Unit" of main menu to localized string "Show Unit Card"
	end if
end will close

on will open theObject
	if name of theObject is "Information" then
		set title of menu item "Show Information" of menu "Unit" of main menu to localized string "Hide Information"
	else if name of theObject is "Preferences" then
		loadPreferencesWindow()
	else if name of theObject is "Unit Card" then
		set title of menu item "Show Unit Card" of menu "Unit" of main menu to localized string "Hide Unit Card"
	end if
end will open

on selection changing theObject
	(* This keeps getting added automatically for some reason *)
end selection changing

on selection changed theObject
	(* And this one, too *)
end selection changed


on addOptions(selectedUnit)
	
	set selectedOptions to selected data rows of table view "Unit Options" of scroll view "Options" of split view "Horizontal" of split view "Vertical" of window "Main"
	
	repeat with s in selectedOptions
		set newOption to make new data item at end of data items of selectedUnit
		set contents of data cell "Unit" of newOption to (contents of data cell "Option" of s)
		set contents of data cell "Warning" of newOption to load image "Empty" --Loading an "empty" image (1x1 pixel transparent) is necessary because not loading an image causes an error message in the log and breaks the program
		set contents of data cell "AppliesToAll" of newOption to (contents of data cell "AppliesToAll" of s)
		set contents of data cell "APperFighter" of newOption to (contents of data cell "APperFighter" of s)
		set contents of data cell "FixedAP" of newOption to (contents of data cell "FixedAP" of s)
		if (contents of data cell "IsOfficer" of s) = true then
			set contents of data cell "Officers" of newOption to 1
		end if
		if (contents of data cell "IsSpecialWeapon" of s) = true then
			set contents of data cell "SpecialWeapons" of newOption to 1
		end if
		if (contents of data cell "IsSpecialist" of s) = true then
			set contents of data cell "Specialists" of newOption to 1
		end if
	end repeat
	
end addOptions


on addOptionToSelectedUnit()
	
	set update views of data source "companyDataSource" to false
	
	set selectedUnit to selected data item of outline view "Company" of scroll view "Company" of view "Company" of split view "Vertical" of window "Main"
	
	if has parent data item of selectedUnit is true then
		set selectedUnit to parent data item of selectedUnit
	end if
	
	addOptions(selectedUnit)
	
	set update views of data source "companyDataSource" to true
	
	addUpUnitStats()
	
	set document edited of window "Main" to true
	
end addOptionToSelectedUnit


on addUnitToCompany()
	
	set unitData to findDataItemForUnit((contents of data cell "Unit" of selected data item of outline view "Available Units" of scroll view "Units" of split view "Horizontal" of split view "Vertical" of window "Main") as string)
	
	set update views of data source "companyDataSource" to false
	
	set newUnit to make new data item at end of data items of data source "companyDataSource"
	tell unitData
		set contents of data cell "Unit" of newUnit to contents of data cell "Unit"
		set contents of data cell "Warning" of newUnit to load image "Empty" --Loading an "empty" image (1x1 pixel transparent) is necessary because not loading an image causes an error message in the log and breaks the program
		set contents of data cell "AP" of newUnit to contents of data cell "AP"
		set contents of data cell "BaseAP" of newUnit to contents of data cell "AP"
		set contents of data cell "APperFighter" of newUnit to contents of data cell "APperFighter"
		set contents of data cell "Type" of newUnit to contents of data cell "Type"
		set contents of data cell "Fighters" of newUnit to contents of data cell "StandardNumber"
		set contents of data cell "MinFighters" of newUnit to contents of data cell "StandardNumber"
		set contents of data cell "MaxFighters" of newUnit to contents of data cell "MaximumNumber"
		set contents of data cell "Officers" of newUnit to (contents of data cell "StndOfficers" of unitData)
		set contents of data cell "MinOfficers" of newUnit to contents of data cell "StndOfficers"
		set contents of data cell "MaxOfficers" of newUnit to contents of data cell "MaxOfficers"
		set contents of data cell "SpecialWeapons" of newUnit to contents of data cell "StndSpecialWeapons"
		set contents of data cell "MinSpecialWeapons" of newUnit to contents of data cell "StndSpecialWeapons"
		set contents of data cell "MaxSpecialWeapons" of newUnit to contents of data cell "MaxSpecialWeapons"
		set contents of data cell "Specialists" of newUnit to (contents of data cell "StndSpecialists" of unitData)
		set contents of data cell "MinSpecialists" of newUnit to contents of data cell "StndSpecialists"
		set contents of data cell "MaxSpecialists" of newUnit to contents of data cell "MaxSpecialists"
	end tell
	
	addOptions(newUnit)
	
	set update views of data source "companyDataSource" to true
	
	addUpUnitStats()
	
	set document edited of window "Main" to true
	
end addUnitToCompany


on addUpUnitStats() --(selectIllegalUnits)
	
	--if selectIllegalUnits is true then set selected data items of outline view "Company" of scroll view "Company" of view "Company" of split view "Vertical" of window "Main" to {}
	
	
	set AP to 0
	repeat with i in data items of data source of outline view "Company" of scroll view "Company" of view "Company" of split view "Vertical" of window "Main"
		
		set totalOptionsAP to 0
		set numberOfOfficers to 0
		set numberOfSpecialWeapons to 0
		set numberOfSpecialists to 0
		
		repeat with o in data items of i
			set optionAP to (contents of data cell "FixedAP" of o) as integer
			
			if contents of data cell "AppliesToAll" of o is true then
				set optionAP to optionAP + ((contents of data cell "APperFighter" of o) as integer) * ((contents of data cell "Fighters" of i) as integer)
			else
				set optionAP to optionAP + ((contents of data cell "APperFighter" of o) as integer) * (((contents of data cell "Officers" of o) as integer) + ((contents of data cell "SpecialWeapons" of o) as integer) + ((contents of data cell "Specialists" of o) as integer))
			end if
			
			set numberOfOfficers to numberOfOfficers + ((contents of data cell "Officers" of o) as integer)
			set numberOfSpecialWeapons to numberOfSpecialWeapons + ((contents of data cell "SpecialWeapons" of o) as integer)
			set numberOfSpecialists to numberOfSpecialists + ((contents of data cell "Specialists" of o) as integer)
			
			set contents of data cell "AP" of o to optionAP
			set totalOptionsAP to totalOptionsAP + optionAP
		end repeat
		
		set unitAP to ((contents of data cell "BaseAP" of i) as integer) + (((contents of data cell "APperFighter" of i) as integer) * (((contents of data cell "Fighters" of i) as integer) - (contents of data cell "MinFighters" of i) as integer)) + totalOptionsAP
		set contents of data cell "AP" of i to unitAP
		set AP to AP + unitAP
		
		set contents of data cell "Officers" of i to numberOfOfficers
		set contents of data cell "SpecialWeapons" of i to numberOfSpecialWeapons
		set contents of data cell "Specialists" of i to numberOfSpecialists
		
		checkUnit(i)
	end repeat
	
	set contents of text field "Current AP" of window "Main" to AP
	
	checkAP()
	
end addUpUnitStats


on changeCase(inText, desiredCase)
	
	if desiredCase is toLowerCase then
		set comparisonString to "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
		set sourceString to "abcdefghijklmnopqrstuvwxyz"
	else
		set comparisonString to "abcdefghijklmnopqrstuvwxyz"
		set sourceString to "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
	end if
	
	set outText to ""
	
	repeat with char in inText
		set x to offset of char in comparisonString
		if x is not 0 then
			set outText to (outText & character x of sourceString) as string
		else
			set outText to (outText & char) as string
		end if
	end repeat
	
	return outText
	
end changeCase


on changeUnitValues(dataCell)
	
	tell view "Company" of split view "Vertical" of window "Main"
		set newValue to contents of stepper dataCell of box "Unit Members"
		
		tell table view "Company" of scroll view "Company"
			set sel to selected data item
			set contents of data cell dataCell of sel to newValue
		end tell
	end tell
	
	addUpUnitStats()
	
	set document edited of window "Main" to true
	
end changeUnitValues


on checkAP()
	tell window "Main"
		set currentAP to contents of text field "Current AP" as integer
		set maxAP to contents of text field "Maximum AP" as integer
		if (contents of stepper "AP Stepper") as integer  maxAP then set contents of stepper "AP Stepper" to maxAP
		
		if currentAP > maxAP then
			set visible of image view "AP Warning" to true
			set image of toolbar item "warning item" of toolbar 1 to load image "Error"
			set label of toolbar item "warning item" of toolbar 1 to localized string "Warning"
			set palette label of toolbar item "warning item" of toolbar 1 to localized string "Warning"
			set tool tip of toolbar item "warning item" of toolbar 1 to localized string "Your company contains illegal units or you have spent too many Army Points"
			if playWarningSound is true then play (load sound warningSound)
		else
			set visible of image view "AP Warning" to false
			set image of toolbar item "warning item" of toolbar 1 to load image "Okay"
			set label of toolbar item "warning item" of toolbar 1 to localized string "Company Okay"
			set palette label of toolbar item "warning item" of toolbar 1 to localized string "Company Okay"
			set tool tip of toolbar item "warning item" of toolbar 1 to localized string "Your company appears to be legal"
		end if
	end tell
end checkAP


on checkUnit(unitData)
	
	set errorLevel to 0
	
	repeat with fighterType in {"Fighters", "Officers", "SpecialWeapons", "Specialists"}
		if (contents of data cell fighterType of unitData) as integer < (contents of data cell ("Min" & fighterType) of unitData) as integer then
			if errorLevel < 1 then set errorLevel to 1
			--set visible of image view ("Alert " & fighterType) of box "Unit Members" of view "Company" of split view "Vertical" of window "Main" to true
		else if (contents of data cell fighterType of unitData) as integer > (contents of data cell ("Max" & fighterType) of unitData) as integer then
			if errorLevel < 2 then set errorLevel to 2
			--set visible of image view ("Alert " & fighterType) of box "Unit Members" of view "Company" of split view "Vertical" of window "Main" to true
			--else
			--set visible of image view ("Alert " & fighterType) of box "Unit Members" of view "Company" of split view "Vertical" of window "Main" to false
		end if
	end repeat
	
	if errorLevel = 1 then
		set (contents of data cell "Warning" of unitData) to load image "Orange small"
		if playWarningSound is true then play (load sound warningSound)
	else if errorLevel = 2 then
		set (contents of data cell "Warning" of unitData) to load image "Red small"
		if playWarningSound is true then play (load sound warningSound)
	else
		set (contents of data cell "Warning" of unitData) to load image "Empty"
	end if
	
end checkUnit


on createToolBar(theObject)
	set documentToolbar to make new toolbar at end with properties {name:"document 
toolbar", identifier:"document toolbar identifier", allows customization:true, auto sizes cells:true, display mode:default display mode, size mode:default size mode}
	
	set default identifiers of documentToolbar to {"info item identifier", "card item identifier", "background item identifier", "space item identifier", "print item identifier", "separator item identifier", "export html item identifier", "export text item identifier", "flexible space item identifer", "warning item identifier"}
	set allowed identifiers of documentToolbar to {"info item identifier", "card item identifier", "background item identifier", "print item identifier", "preferences item identifier", "export html item identifier", "export text item identifier", "warning item identifier", "flexible space item identifer", "space item identifier", "separator item identifier", "customize toolbar item identifer"}
	
	make new toolbar item at end of toolbar items of documentToolbar with properties {identifier:"info item identifier", name:"info item", label:localized string "Show Info", palette label:localized string "Show Info", tool tip:localized string "Show information about the selected unit", image name:"Info"}
	
	make new toolbar item at end of toolbar items of documentToolbar with properties {identifier:"warning item identifier", name:"warning item", label:localized string "Company Okay", palette label:localized string "Company Okay", tool tip:localized string "Your company appears to be legal", image name:"Okay"}
	
	make new toolbar item at end of toolbar items of documentToolbar with properties {identifier:"card item identifier", name:"card item", label:localized string "Show Unit Card", palette label:localized string "Show Unit Card", tool tip:localized string "Show the stat card for the selected unit", image name:"Card"}
	
	make new toolbar item at end of toolbar items of documentToolbar with properties {identifier:"background item identifier", name:"background item", label:localized string "Edit Unit Background", palette label:localized string "Edit Unit Background", tool tip:localized string "Edit background information for the selected unit", image name:"Background"}
	
	make new toolbar item at end of toolbar items of documentToolbar with properties {identifier:"export html item identifier", name:"export html item", label:localized string "Export to Web", palette label:localized string "Export Company to Web", tool tip:localized string "Export the company to a web page", image name:"Export HTML"}
	
	make new toolbar item at end of toolbar items of documentToolbar with properties {identifier:"export text item identifier", name:"export text item", label:localized string "Export  to Text File", palette label:localized string "Export Company to Text", tool tip:localized string "Export the company to a text file", image name:"Export Text"}
	
	make new toolbar item at end of toolbar items of documentToolbar with properties {identifier:"preferences item identifier", name:"preferences item", label:localized string "Preferences", palette label:localized string "Preferences", tool tip:localized string "Show the preferences", image name:"Preferences"}
	
	make new toolbar item at end of toolbar items of documentToolbar with properties {identifier:"help item identifier", name:"help item", label:localized string "Help", palette label:localized string "Help", tool tip:localized string "Show help for M.Ind", image name:"Help"}
	
	set toolbar of theObject to documentToolbar
	
end createToolBar


on emptyOptionsList()
	
	set contents of table view "Unit Options" of scroll view "Options" of split view "Horizontal" of split view "Vertical" of window "Main" to {}
	
end emptyOptionsList


on enableAddButtonAndMenu(enable)
	
	set enabled of button "Add" of window "Main" to enable
	--set enabled of menu item "Add" of menu "Unit" of main menu to enable
	
end enableAddButtonAndMenu


on enableRemoveButtonAndMenu(enable)
	
	tell window "Main"
		set enabled of button "Remove Unit" to enable
		tell box "Unit Members" of view "Company" of split view "Vertical"
			set enabled of button "Standard Unit" to enable
			set enabled of button "Maximize Unit" to enable
		end tell
	end tell
	
	tell menu "Unit" of main menu
		set enabled of menu item "Remove Unit" to enable
		set enabled of menu item "Maximize Unit" to enable
		set enabled of menu item "Standard Unit" to enable
	end tell
	
end enableRemoveButtonAndMenu


on exportCompany(exportFormat)
	
	tell save panel
		set prompt to localized string "Export"
		set required file type to exportFormat
	end tell
	
	if saveFileName = "" then
		set fileName to title of current menu item of popup button "Armies" of window "Main" & " " & contents of text field "Current AP" of window "Main" & " AP." & exportFormat
	else
		set fileName to (characters 1 through -5 of saveFileName) & "." & exportFormat
	end if
	
	display save panel attached to window "Main" for file types {exportFormat} in directory "~" with file name fileName
	
end exportCompany


on findDataItemForUnit(unitToFind)
	
	set foundItem to ""
	
	repeat with topItem in data items of data source "unitDataSource"
		repeat with unit in data items of topItem
			if unitToFind = contents of data cell "Unit" of unit then
				set foundItem to unit
				exit repeat
			end if
		end repeat
	end repeat
	
	return foundItem
	
end findDataItemForUnit


on getUnitData(unitName)
	
	tell progress indicator "Army Progress" of window "Main" to start
	
	if has parent data item of selected data item of outline view "Available Units" of scroll view "Units" of split view "Horizontal" of split view "Vertical" of window "Main" = true then
		
		set unitData to findDataItemForUnit(unitName)
		
		set stndFighters to (contents of data cell "StandardNumber" of unitData) as string
		set maxFighters to (contents of data cell "MaximumNumber" of unitData) as string
		set stndSpecialWeapons to (contents of data cell "StndSpecialWeapons" of unitData) as string
		set maxSpecialWeapons to (contents of data cell "MaxSpecialWeapons" of unitData) as string
		set stndSpecialists to (contents of data cell "StndSpecialists" of unitData) as string
		set maxSpecialists to (contents of data cell "MaxSpecialists" of unitData) as string
		set stndOfficers to (contents of data cell "StndOfficers" of unitData) as string
		set maxOfficers to (contents of data cell "MaxOfficers" of unitData) as string
		
		tell table view "Unit Data" of scroll view "Unit Data" of split view "Drawer" of drawer "Information" of window "Main"
			set contents to {{localized string "Fighters", stndFighters, maxFighters}, {localized string "Special Weapons", stndSpecialWeapons, maxSpecialWeapons}, {localized string "Specialists", stndSpecialists, maxSpecialists}, {localized string "Officers", stndOfficers, maxOfficers}}
			set contents of header cell of table column 1 to unitName
			update
		end tell
		
		setUnitImage((resource path of main bundle) & "/" & unitName & ".jpg", (resource path of main bundle) & "/" & title of current menu item of popup button "Armies" of window "Main" & ".png")
		
		showOptions(unitData)
		
		enableAddButtonAndMenu(true)
		enableRemoveButtonAndMenu(false)
	else
		enableAddButtonAndMenu(false)
		tell table view "Unit Data" of scroll view "Unit Data" of split view "Drawer" of drawer "Information" of window "Main"
			set contents to {}
			set contents of header cell of table column 1 to ""
			update
		end tell
		
		setUnitImage((resource path of main bundle) & "/" & title of current menu item of popup button "Armies" of window "Main" & ".png", "")
		
		emptyOptionsList()
	end if
	
	tell progress indicator "Army Progress" of window "Main" to stop
	
end getUnitData


on getUnitType(unitType)
	
	set n to 1
	set stars to "*"
	repeat until n = unitType
		set stars to stars & "*"
		set n to n + 1
	end repeat
	
	return stars
	
end getUnitType


on loadArmies()
	
	set AppleScript's text item delimiters to {return}
	set resourceDir to resource path of main bundle
	set armyFiles to text items of (do shell script "ls -1 " & quoted form of resourceDir & "/*.dat")
	
	tell window "Main"
		if length of armyFiles > 0 then
			set enabled of popup button "Armies" to false
			set title of first menu item of menu of popup button "Armies" to localized string "Loading armies "
			repeat with army in armyFiles
				set AppleScript's text item delimiters to "/"
				set army to last text item of army
				set AppleScript's text item delimiters to ""
				set army to characters 1 thru -5 of army as string
				
				make new menu item at end of menu items of menu of popup button "Armies" with properties {title:(localized string army)}
				
				loadUnits(army) of me
			end repeat
			
			delete first menu item of menu of popup button "Armies"
			set enabled of popup button "Armies" to true
		end if
	end tell
	
end loadArmies


on loadPreferencesWindow()
	
	tell tab view "Preferences" of window "Preferences"
		tell tab view item "General"
			if allowUnderstrengthUnits is true then
				set state of button "Allow Understrength Units" to 1
			else
				set state of button "Allow Understrength Units" to 0
			end if
			if playWarningSound is true then
				set state of button "Warning Sound" to 1
			else
				set state of button "Warning Sound" to 0
			end if
			
			set current menu item of popup button "Warning Sounds" to menu item warningSound of menu of popup button "Warning Sounds"
			set enabled of popup button "Warning Sounds" to playWarningSound
		end tell
		
		tell tab view item "Export"
			if lineEnding is (ASCII character 13) & (ASCII character 10) then
				set state of button "Windows Text Files" to 1
			else
				set state of button "Windows Text Files" to 0
			end if
			
			tell matrix "HTMLflavor"
				if htmlFlavor = "xhtml1" then
					set current row to 1
				else if htmlFlavor = "html401" then
					set current row to 2
				else if htmlFlavor = "html32" then
					set current row to 3
				else
					set current row to 4
				end if
			end tell
			
		end tell
		
	end tell
	
end loadPreferencesWindow


on loadUnits(armyName)
	
	tell progress indicator "Army Progress" of window "Main" to start
	
	setUnitImage((resource path of main bundle) & "/" & armyName & ".png", "")
	
	-- Read army file
	set armyFile to (resource path of main bundle) & "/" & armyName & ".dat"
	
	set AppleScript's text item delimiters to {return}
	
	set armyList to text items of (do shell script "cat " & quoted form of armyFile)
	set AppleScript's text item delimiters to ","
	
	repeat with unitLine in armyList
		if unitLine is not "" then
			set currentUnit to text items of unitLine
			if first item of currentUnit = "1" then --heroes
				set parentItem to data item ((first item of currentUnit) as integer) of data source "unitDataSource"
				set childItem to make new data item at end of data items of parentItem
				set contents of data cell "Army" of childItem to localized string armyName from table "Units"
				set contents of data cell "Unit" of childItem to localized string (item 2 of currentUnit) from table "Units"
				set contents of data cell "EnglishUnitName" of childItem to item 2 of currentUnit
				set contents of data cell "Type" of childItem to getUnitType((item 3 of currentUnit) as integer)
				set contents of data cell "AP" of childItem to item 4 of currentUnit
				set contents of data cell "StandardNumber" of childItem to item 5 of currentUnit
				set contents of data cell "MaximumNumber" of childItem to item 6 of currentUnit
				set contents of data cell "NumbersBonus" of childItem to item 7 of currentUnit
				if item 8 of currentUnit = "True" then
					set contents of data cell "IsOfficer" of childItem to true
				else
					set contents of data cell "IsOfficer" of childItem to false
				end if
				
			else if first item of currentUnit = "2" then --infantry
				set parentItem to data item ((first item of currentUnit) as integer) of data source "unitDataSource"
				set childItem to make new data item at end of data items of parentItem
				set contents of data cell "Army" of childItem to localized string armyName from table "Units"
				set contents of data cell "Unit" of childItem to localized string (item 2 of currentUnit) from table "Units"
				set contents of data cell "EnglishUnitName" of childItem to item 2 of currentUnit
				set contents of data cell "Type" of childItem to getUnitType((item 3 of currentUnit) as integer)
				set contents of data cell "AP" of childItem to item 4 of currentUnit
				set contents of data cell "StandardNumber" of childItem to item 5 of currentUnit
				set contents of data cell "MaximumNumber" of childItem to item 6 of currentUnit
				set contents of data cell "NumbersBonus" of childItem to item 7 of currentUnit
				set contents of data cell "APperFighter" of childItem to item 8 of currentUnit
				set contents of data cell "StndSpecialists" of childItem to item 9 of currentUnit
				set contents of data cell "MaxSpecialists" of childItem to item 10 of currentUnit
				set contents of data cell "StndSpecialWeapons" of childItem to item 11 of currentUnit
				set contents of data cell "MaxSpecialWeapons" of childItem to item 12 of currentUnit
				set contents of data cell "StndOfficers" of childItem to item 13 of currentUnit
				set contents of data cell "MaxOfficers" of childItem to item 14 of currentUnit
				
			else if first item of currentUnit = "3" then --combat striders
				set parentItem to data item ((first item of currentUnit) as integer) of data source "unitDataSource"
				set childItem to make new data item at end of data items of parentItem
				set contents of data cell "Army" of childItem to localized string armyName from table "Units"
				set contents of data cell "Unit" of childItem to localized string (item 2 of currentUnit) from table "Units"
				set contents of data cell "EnglishUnitName" of childItem to item 2 of currentUnit
				set contents of data cell "Type" of childItem to getUnitType((item 3 of currentUnit) as integer)
				set contents of data cell "AP" of childItem to item 4 of currentUnit
				set contents of data cell "StandardNumber" of childItem to item 5 of currentUnit
				set contents of data cell "MaximumNumber" of childItem to item 6 of currentUnit
				set contents of data cell "NumbersBonus" of childItem to item 7 of currentUnit
				set contents of data cell "APperFighter" of childItem to item 8 of currentUnit
				set contents of data cell "StndOfficers" of childItem to item 9 of currentUnit
				set contents of data cell "MaxOfficers" of childItem to item 10 of currentUnit
				set contents of data cell "StndSpecialists" of childItem to 0
				set contents of data cell "MaxSpecialists" of childItem to 0
				set contents of data cell "StndSpecialWeapons" of childItem to 0
				set contents of data cell "MaxSpecialWeapons" of childItem to 0
				
			else if first item of currentUnit = "4" then --vehicles
				-- stats of vehicles are currently unknown
				
			else if first item of currentUnit = "10" then --options/equipment
				set newItem to make new data item at end of data items of data source "optionsDataSource"
				set contents of data cell "Unit" of newItem to item 2 of currentUnit
				set contents of data cell "OptionName" of newItem to item 3 of currentUnit
				set contents of data cell "FixedAP" of newItem to item 4 of currentUnit
				set contents of data cell "APperFighter" of newItem to item 5 of currentUnit
				set contents of data cell "AppliesToAll" of newItem to true
				
			else if first item of currentUnit = "20" then --officers
				set newItem to make new data item at end of data items of data source "optionsDataSource"
				set contents of data cell "Unit" of newItem to item 2 of currentUnit
				set contents of data cell "OptionName" of newItem to item 3 of currentUnit
				set contents of data cell "FixedAP" of newItem to item 4 of currentUnit
				set contents of data cell "APperFighter" of newItem to item 5 of currentUnit
				set contents of data cell "NumbersBonus" of newItem to item 6 of currentUnit
				set contents of data cell "IsOfficer" of newItem to true
				set contents of data cell "AppliesToAll" of newItem to false
				
			else if first item of currentUnit = "21" then --special weapons
				set newItem to make new data item at end of data items of data source "optionsDataSource"
				set contents of data cell "Unit" of newItem to item 2 of currentUnit
				set contents of data cell "OptionName" of newItem to item 3 of currentUnit
				set contents of data cell "FixedAP" of newItem to item 4 of currentUnit
				set contents of data cell "APperFighter" of newItem to item 5 of currentUnit
				set contents of data cell "IsSpecialWeapon" of newItem to true
				set contents of data cell "AppliesToAll" of newItem to false
				
			else if first item of currentUnit = "22" then --specialists
				set newItem to make new data item at end of data items of data source "optionsDataSource"
				set contents of data cell "Unit" of newItem to item 2 of currentUnit
				set contents of data cell "OptionName" of newItem to item 3 of currentUnit
				set contents of data cell "FixedAP" of newItem to item 4 of currentUnit
				set contents of data cell "APperFighter" of newItem to item 5 of currentUnit
				set contents of data cell "IsSpecialist" of newItem to true
				set contents of data cell "AppliesToAll" of newItem to false
			end if
		end if
	end repeat
	
	set AppleScript's text item delimiters to ""
	
	tell progress indicator "Army Progress" of window "Main" to stop
	
end loadUnits


on maximizeOrStandardUnit(maximize)
	
	tell box "Unit Members" of view "Company" of split view "Vertical" of window "Main"
		if maximize = true then
			repeat with itemName in {"Fighters", "SpecialWeapons"}
				set contents of stepper itemName to maximum value of stepper itemName
				syncTextAndStepper(itemName, true) of me
			end repeat
		else
			repeat with itemName in {"Fighters", "SpecialWeapons"}
				set contents of stepper itemName to minimum value of stepper itemName
				syncTextAndStepper(itemName, true) of me
			end repeat
		end if
	end tell
	
end maximizeOrStandardUnit


on newCompany()
	
	if document edited of window "Main" = true then display alert (localized string "Do you want to save the changes you made to the company?") default button (localized string "Save") alternate button (localized string "Cancel") other button (localized string "Don't Save") message (localized string "Your changes will be lost if you don't save them.") as warning attached to window "Main"
	
end newCompany


on openCompany()
	
	newCompany()
	display open panel for file types {"mind"} in directory "~" attached to window "Main"
	
end openCompany


on padString(inString, lineLength)
	
	repeat until length of inString  lineLength
		set inString to inString & " "
	end repeat
	
	return inString
	
end padString


on RemoveUnitFromCompany()
	
	tell table view "Company" of scroll view "Company" of view "Company" of split view "Vertical" of window "Main"
		set sel to selected data items
		tell data source
			repeat with s in sel
				delete s
			end repeat
		end tell
		set selected data items to {}
	end tell
	
	addUpUnitStats()
	
	set document edited of window "Main" to true
	
end RemoveUnitFromCompany


on replacePercents(inText, percentText, replacementText)
	
	set AppleScript's text item delimiters to percentText
	set splitText to every text item of inText
	set AppleScript's text item delimiters to replacementText
	set inText to splitText as string
	set AppleScript's text item delimiters to ""
	
	return inText
	
end replacePercents


on resetCompany()
	
	set contents of table view "Company" of scroll view "Company" of view "Company" of split view "Vertical" of window "Main" to {}
	addUpUnitStats()
	set document edited of window "Main" to false
	
end resetCompany


on saveCompany()
	
	if saveFileName = "" then
		saveCompanyAs()
	else
		writeCompanyToFile()
	end if
	
end saveCompany


on saveCompanyAs()
	
	tell save panel
		set prompt to localized string "Save"
		set required file type to "mind"
	end tell
	
	if saveFileName = "" then
		set fileName to title of current menu item of popup button "Armies" of window "Main" & " " & contents of text field "Maximum AP" of window "Main" & " AP.mind"
		set filePath to "~/Documents/"
	else
		set AppleScript's text item delimiters to "/"
		set fileName to last text item of saveFileName
		set filePath to text items 1 thru -2 of saveFileName
		set AppleScript's text item delimiters to ""
	end if
	
	display save panel attached to window "Main" for file types {"mind"} in directory filePath with file name fileName
	
end saveCompanyAs


on selectUnit(unitData)
	
	--set selected data item of outline view "Available Units" of scroll view "Units" of split view "Horizontal" of split view "Vertical" of window "Main" to data item id 198 of data item id 195 of data source id 190
	
	(*
	set currentUnit to contents of data cell "Unit" of unitData
	--tell outline view "Available Units" of scroll view "Units" of split view "Horizontal" of split view "Vertical" of window "Main"
	repeat with unitCategory in data items of data source "unitDataSource"
		repeat with unit in data items of unitCategory
			set unitName to contents of data cell "Unit" of unit
			if (unitName as string) is (currentUnit as string) then
				set selected data item of outline view "Available Units" of scroll view "Units" of split view "Horizontal" of split view "Vertical" of window "Main" to unit
				exit repeat
			end if
			--log currentUnit as string
			--log unit
		end repeat
	end repeat
	--end tell
	*)
	
end selectUnit


on setUnitImage(imageFileName, alternativeImage)
	
	if alternativeImage = "" then set alternativeImage to "M.Ind"
	
	tell split view "Drawer" of drawer "Information" of window "Main"
		try
			set image of image view "Unit Picture" to load image imageFileName
		on error number -1700
			try
				set image of image view "Unit Picture" to load image alternativeImage
			on error number -1700
				set image of image view "Unit Picture" to load image ("M.Ind")
			end try
		end try
	end tell
	try
		set image of image view "Unit Card" of window "Unit Card" to image of image view "Unit Picture" of split view "Drawer" of drawer "Information" of window "Main"
	end try
end setUnitImage


on setUnitMembers(unitData)
	
	tell box "Unit Members" of view "Company" of split view "Vertical" of window "Main"
		if has parent data item of unitData is true then
			
			repeat with fighterType in {"Fighters", "Officers", "SpecialWeapons", "Specialists"}
				if (contents of data cell fighterType of unitData) as string  "" then
					set enabled of text field fighterType to true
					set enabled of stepper fighterType to true
					setUnitMemberSteppers(fighterType, contents of data cell fighterType of unitData, parent data item of unitData) of me
					set contents of text field fighterType to (contents of data cell fighterType of unitData) as integer
				else
					set enabled of text field fighterType to false
					set enabled of stepper fighterType to false
					set contents of text field fighterType to ""
				end if
			end repeat
			
		else
			
			set enabled of text field "Fighters" to true
			set enabled of stepper "Fighters" to true
			setUnitMemberSteppers("Fighters", contents of data cell "Fighters" of unitData, unitData) of me
			set contents of text field "Fighters" to (contents of data cell "Fighters" of unitData) as integer
			
			repeat with fighterType in {"Officers", "SpecialWeapons", "Specialists"}
				set enabled of text field fighterType to false
				set enabled of stepper fighterType to false
				set contents of text field fighterType to ""
			end repeat
			
			set contents of text view "Unit Background" of scroll view "Unit Background" of window "Unit Background" of me to (contents of data cell "UnitBackground" of unitData) as string
			
		end if
	end tell
	
end setUnitMembers


on setUnitMemberSteppers(fighterType, currentValue, unitData)
	
	tell box "Unit Members" of view "Company" of split view "Vertical" of window "Main"
		
		if allowUnderstrengthUnits is true then
			if fighterType is "Fighters" then
				set minimum value of stepper fighterType to 1
			else
				set minimum value of stepper fighterType to 0
			end if
		else
			set minimum value of stepper fighterType to (contents of data cell ("Min" & fighterType) of unitData) as integer
		end if
		
		set maximum value of stepper fighterType to (contents of data cell ("Max" & fighterType) of unitData) as integer
		set contents of stepper fighterType to currentValue as integer
	end tell
	
end setUnitMemberSteppers


on showOptions(unitData)
	
	set unitOptions to make new data source at end of data sources with properties {name:"Unit Options"}
	
	-- Create the data columns for unit data
	tell unitOptions
		repeat with columnName in {"Option", "AP", "APperFighter", "FixedAP", "IsOfficer", "IsSpecialWeapon", "IsSpecialist", "AppliesToAll"}
			make new data column at end of data columns with properties {name:columnName}
		end repeat
	end tell
	
	repeat with option in data items of data source "optionsDataSource"
		if contents of data cell "Unit" of option = (contents of data cell "EnglishUnitName" of unitData) as string then
			set newItem to make new data row at end of data rows of unitOptions
			set contents of data cell "Option" of newItem to contents of data cell "OptionName" of option
			
			set fixedAP to (contents of data cell "FixedAP" of option) as integer
			set apPerFighter to (contents of data cell "APperFighter" of option) as integer
			set contents of data cell "FixedAP" of newItem to fixedAP
			set contents of data cell "APperFighter" of newItem to apPerFighter
			if contents of data cell "AppliesToAll" of option = true then
				set optionAP to fixedAP + apPerFighter * ((contents of data cell "StandardNumber" of unitData) as integer)
			else if contents of data cell "IsOfficer" of option = true or contents of data cell "IsSpecialWeapon" of option = true or contents of data cell "IsSpecialist" of option = true then
				set optionAP to fixedAP + apPerFighter
			else
				set optionAP to fixedAP
			end if
			set contents of data cell "AP" of newItem to (fixedAP as string) & "/" & apPerFighter as string
			set contents of data cell "IsOfficer" of newItem to contents of data cell "IsOfficer" of option
			set contents of data cell "IsSpecialWeapon" of newItem to contents of data cell "IsSpecialWeapon" of option
			set contents of data cell "IsSpecialist" of newItem to contents of data cell "IsSpecialist" of option
			set contents of data cell "AppliesToAll" of newItem to contents of data cell "AppliesToAll" of option
		end if
	end repeat
	
	-- Assign the data source to the table view
	tell scroll view "Options" of split view "Horizontal" of split view "Vertical" of window "Main"
		set data source of table view "Unit Options" to unitOptions
		set selected data rows of table view "Unit Options" to {}
	end tell
	
end showOptions


on showUnits(armyName)
	
	tell progress indicator "Army Progress" of window "Main" to start
	
	set armyUnits to make new data source at end of data sources with properties {name:"Army Units"}
	
	-- Create the data columns for unit data
	tell armyUnits
		repeat with columnName in {"Unit", "Type", "AP"}
			make new data column at end of data columns with properties {name:columnName}
		end repeat
	end tell
	
	-- Set up categories
	set parentItem to make new data item at end of data items of armyUnits with properties {name:"Heroes"}
	set contents of data cell "Unit" of parentItem to localized string "Heroes" from table "Units"
	
	set parentItem to make new data item at end of data items of armyUnits with properties {name:"Infantry"}
	set contents of data cell "Unit" of parentItem to localized string "Infantry" from table "Units"
	
	set parentItem to make new data item at end of data items of armyUnits with properties {name:"Combat Striders"}
	set contents of data cell "Unit" of parentItem to localized string "Combat Striders" from table "Units"
	
	set parentItem to make new data item at end of data items of armyUnits with properties {name:"Vehicles"}
	set contents of data cell "Unit" of parentItem to localized string "Vehicles" from table "Units"
	
	
	repeat with unitType in data items of data source "unitDataSource"
		repeat with unit in data items of unitType
			if contents of data cell "Army" of unit = armyName then
				set childItem to make new data item at end of data items of data item (contents of data cell "Unit" of unitType) of armyUnits
				set contents of data cell "Unit" of childItem to contents of data cell "Unit" of unit
				set contents of data cell "Type" of childItem to contents of data cell "Type" of unit
				set contents of data cell "AP" of childItem to contents of data cell "AP" of unit
			end if
		end repeat
	end repeat
	
	-- Assign the data source to the outline view
	set data source of outline view "Available Units" of scroll view "Units" of split view "Horizontal" of split view "Vertical" of window "Main" to armyUnits
	
	tell progress indicator "Army Progress" of window "Main" to stop
	
end showUnits


on syncTextAndStepper(nameOfItem, stepperClicked)
	
	tell box "Unit Members" of view "Company" of split view "Vertical" of window "Main"
		if stepperClicked = true then
			set contents of text field nameOfItem to contents of stepper nameOfItem
		else
			set t to contents of text field nameOfItem as integer
			if t > maximum value of stepper nameOfItem then
				set t to maximum value of stepper nameOfItem
			else if t < minimum value of stepper nameOfItem then
				set t to minimum value of stepper nameOfItem
			end if
			set contents of stepper nameOfItem to t
			set contents of text field nameOfItem to t as integer
		end if
	end tell
	
	changeUnitValues(nameOfItem)
	
end syncTextAndStepper


on writeCompanyToFile()
	
	display dialog "This part of the program has not been implemented yet. You would have saved your company to the file " & saveFileName & "." buttons {"OK"} with icon "M.Ind document"
	
	(*
	set fileNumber to open for access (POSIX file saveFileName) with write permission
	set eof of fileNumber to 0
	write contents of table view "Company" of scroll view "Company" of view "Company" of split view "Vertical" of window "Main" to fileNumber
	close access fileNumber
	
	set document edited of window "Main" to false
	*)
end writeCompanyToFile


on writeCompanyToHTML(fileName)
	
	set fileNumber to open for access (POSIX file fileName) with write permission
	set eof of fileNumber to 0
	
	tell window "Main"
		
		if htmlFlavor = "html32" then
			write "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">" & lineEnding & "<html>" & lineEnding & "<head>" & lineEnding & "<meta http-equiv=\"Content-Type\" content=\"text/html;charset=utf-8\">" & lineEnding to fileNumber
		else if htmlFlavor = "html401" then
			write "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">" & lineEnding & "<html>" & lineEnding & "<head>" & lineEnding & "<meta http-equiv=\"Content-Type\" content=\"text/html;charset=utf-8\">" & lineEnding to fileNumber
		else if htmlFlavor = "xhtml1" then
			write "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">" & lineEnding & "<html xmlns=\"http://www.w3.org/1999/xhtml\">" & lineEnding & "<head>" & lineEnding & "<meta http-equiv=\"Content-Type\" content=\"text/html;charset=utf-8\"/>" & lineEnding to fileNumber
		else
			write "<html>" & lineEnding & "<head>" & lineEnding to fileNumber
		end if
		
		set docTitle to localized string "%1: %2 A.P."
		set docTitle to replacePercents(docTitle, "%1", (title of current menu item of popup button "Armies")) of me
		set docTitle to replacePercents(docTitle, "%2", (contents of text field "Current AP") as string) of me
		
		write "<title>" & docTitle & "</title>" & lineEnding & "</head>" & lineEnding & "<body>" & lineEnding & "<h1>" to fileNumber
		
		write docTitle to fileNumber
		
		write "</h1>" & lineEnding & "<table>" & lineEnding & "<tr><th>" to fileNumber
		
		-- The extra "as string" in the following "write" lines is necesary, else http://validator.w3.org/
		-- complains that each character in the localized string is not in the SGML character set
		write (localized string "Unit Type") as string to fileNumber
		write "</th> <th>" to fileNumber
		
		write (localized string "A.P.") as string to fileNumber
		write "</th> <th>" to fileNumber
		
		write (localized string "Fighters") as string to fileNumber
		write "</th> <th>" to fileNumber
		
		write (localized string "Officers") as string to fileNumber
		write "</th> <th>" to fileNumber
		
		write (localized string "Special Weapons") as string to fileNumber
		write "</th> <th>" to fileNumber
		
		write (localized string "Specialists") as string to fileNumber
		write "</th><th>" to fileNumber
		
		write (localized string "Options") as string to fileNumber
		write "</th></tr>" to fileNumber
		
		repeat with unitData in data items of data source of table view "Company" of scroll view "Company" of view "Company" of split view "Vertical"
			write "<tr><td><b>" & ((contents of data cell "Unit" of unitData) as string) & ("</b></td> <td>") & ((contents of data cell "AP" of unitData) as string) & ("</td> <td>") & ((contents of data cell "Fighters" of unitData) as integer) & ("</td> <td>") & ((contents of data cell "Officers" of unitData) as integer) & ("</td> <td>") & ((contents of data cell "SpecialWeapons" of unitData) as integer) & ("</td> <td>") & ((contents of data cell "Specialists" of unitData) as integer) & "</td> <td>" to fileNumber
			
			if has data items of unitData is true then
				repeat with selectedOption in data items of unitData
					repeat with fighterType in {"Fighters", "Officers", "SpecialWeapons", "Specialists"}
						if (contents of data cell fighterType of selectedOption) as integer  0 then
							set numberOfOption to (contents of data cell fighterType of selectedOption) as integer
							exit repeat
						end if
					end repeat
					set unitLine to (localized string ("%1 (%2)"))
					set unitLine to replacePercents(unitLine, "%1", (contents of data cell "Unit" of selectedOption)) of me
					set unitLine to replacePercents(unitLine, "%2", numberOfOption as string) of me
					if htmlFlavor = "xhtml1" then
						set unitLine to unitLine & "<br/>"
					else
						set unitLine to unitLine & "<br>"
					end if
					write unitLine to fileNumber
				end repeat
			end if
			
			write ("</td></tr>") & lineEnding to fileNumber
			
			if ((contents of data cell "UnitBackground" of unitData) as string)  "" then
				set unitLine to ((localized string "<i>Unit Background:</i> %1") as string)
				set unitLine to "<tr><td colspan=\"6\">" & replacePercents(unitLine, "%1", (contents of data cell "UnitBackground" of unitData)) & " " & ((contents of data cell "UnitBackground" of unitData) as string) & "</td></tr>" & lineEnding
				write unitLine to fileNumber
			end if
		end repeat
		
		write "</table>" & lineEnding & "<p><i>" to fileNumber
		set creator to (localized string "Company created using <a href=\"%1\">M.Ind %2</a>")
		set creator to replacePercents(creator, "%1", MIndWebSite) of me
		if htmlFlavor = "html401" or htmlFlavor = "xhtml1" then
			set creator to replacePercents(creator, "%2", MIndVersionHTML) of me
		else
			set creator to replacePercents(creator, "%2", MIndVersion) of me
		end if
		write creator to fileNumber
		write "</i></p>" & lineEnding & "</body>" & lineEnding & "</html>" to fileNumber
	end tell
	
	close access fileNumber
	
end writeCompanyToHTML


on writeCompanyToText(fileName)
	
	set fileNumber to open for access (POSIX file fileName) with write permission
	set eof of fileNumber to 0
	
	tell window "Main"
		
		set unitLine to localized string "%1: %2 A.P."
		set unitLine to replacePercents(unitLine, "%1", (title of current menu item of popup button "Armies")) of me
		set unitLine to replacePercents(unitLine, "%2", (contents of text field "Current AP") as string) of me & lineEnding & lineEnding
		write unitLine to fileNumber
		
		set unitNameLength to length of (localized string "Unit Type")
		repeat with dataRow in data items of data source of table view "Company" of scroll view "Company" of view "Company" of split view "Vertical"
			if length of ((contents of data cell "Unit" of dataRow) as string) > unitNameLength then set unitNameLength to length of ((contents of data cell "Unit" of dataRow) as string)
		end repeat
		
		set apPos to unitNameLength + 6
		set fighterPos to apPos + (length of (localized string "A.P.")) + 4
		set officerPos to fighterPos + (length of (localized string "Fighters")) + 4
		set specWeaponPos to officerPos + (length of (localized string "Officers")) + 4
		set specialistPos to specWeaponPos + (length of (localized string "Special Weapons")) + 4
		
		set unitLine to padString(localized string "Unit Type", apPos - (length of (localized string "A.P.")) / 2) of me & (localized string "A.P.")
		set unitLine to padString(unitLine, fighterPos - (length of (localized string "Fighters")) / 2) of me & (localized string "Fighters")
		set unitLine to padString(unitLine, officerPos - (length of (localized string "Officers")) / 2) of me & (localized string "Officers")
		set unitLine to padString(unitLine, specWeaponPos - (length of (localized string "Special Weapons")) / 2) of me & (localized string "Special Weapons")
		set unitLine to padString(unitLine, specialistPos - (length of (localized string "Specialists")) / 2) of me & (localized string "Specialists")
		
		set unitLine to unitLine & lineEnding
		write unitLine to fileNumber
		
		repeat with unitData in data items of data source of table view "Company" of scroll view "Company" of view "Company" of split view "Vertical"
			set unitLine to ((contents of data cell "Unit" of unitData) as string)
			set unitLine to padString(unitLine, apPos - (length of ((contents of data cell "AP" of unitData) as string)) / 2) of me & ((contents of data cell "AP" of unitData) as string)
			set unitLine to padString(unitLine, fighterPos) of me & ((contents of data cell "Fighters" of unitData) as integer)
			set unitLine to padString(unitLine, officerPos) of me & ((contents of data cell "Officers" of unitData) as integer)
			set unitLine to padString(unitLine, specWeaponPos) of me & ((contents of data cell "SpecialWeapons" of unitData) as integer)
			set unitLine to padString(unitLine, specialistPos) of me & ((contents of data cell "Specialists" of unitData) as integer)
			
			set unitLine to unitLine & lineEnding
			write unitLine to fileNumber
			if has data items of unitData is true then
				set unitLine to (localized string "   Options") & lineEnding
				write unitLine to fileNumber
				repeat with selectedOption in data items of unitData
					repeat with fighterType in {"Fighters", "Officers", "SpecialWeapons", "Specialists"}
						if (contents of data cell fighterType of selectedOption) as integer  0 then
							set numberOfOption to (contents of data cell fighterType of selectedOption) as integer
							exit repeat
						end if
					end repeat
					set unitLine to (localized string ("   %1 (%2)") & lineEnding)
					set unitLine to replacePercents(unitLine, "%1", (contents of data cell "Unit" of selectedOption)) of me
					set unitLine to replacePercents(unitLine, "%2", numberOfOption as string) of me
					write unitLine to fileNumber
				end repeat
			end if
			if (contents of data cell "UnitBackground" of unitData)  "" then
				set unitLine to (localized string ("  Unit Background: %1"))
				set unitLine to replacePercents(unitLine, "%1", (contents of data cell "UnitBackground" of unitData))
				write unitLine to fileNumber
			end if
		end repeat
		
		set creator to lineEnding & lineEnding & lineEnding & (localized string "Company created using M.Ind %1")
		set creator to replacePercents(creator, "%1", MIndVersion) of me
		write creator to fileNumber
		
	end tell
	
	close access fileNumber
	
end writeCompanyToText

